package net.varek.rs2d.model;

/**
 * A mob represents a mobile entity.
 */
public abstract class Mob extends Entity {
	public static final int[][] MOB_SPRITES = new int[][]{
			{14,  0,  2},
			{12, -1,  4},
			{10,  8,  6}
	};
	public static final int[] CLIENT_SPRITES = new int[] {};
		private transient int mobSprite = -1, combatLevel = 3;
	private transient PathHandler pathHandler;
	private transient ViewArea viewArea;
	
	public Mob() {
		 pathHandler = new PathHandler(this);
		 viewArea = new ViewArea(this);
	}
	
	public PathHandler getPathHandler() { 
		return pathHandler; 
	}
	
	public void updatePosition() { 
		pathHandler.updatePosition(); 
	}
	
	public void resetPath() { 
		pathHandler.resetPath();
	}
	
	public ViewArea getViewArea() { 
		return viewArea; 
	}
	
	public int getCombatLevel() { 
		return combatLevel; 
	}
	
	public void setCombatLevel(int level) {
		combatLevel = level;
	}
	
	public void setLocation(Point p) {	
		setLocation(p, false);	
	}
	
	public void setLocation(Point p, boolean teleported) {
		if (!teleported) {
			updateSprite(p);
		}
		super.setLocation(p);
	}

	public void updateSprite(Point newLocation) {
		int xIndex, yIndex;
		int newSprite;
		xIndex = getLocation().getX() - newLocation.getX() + 1;
		yIndex = getLocation().getY() - newLocation.getY() + 1;
		if(xIndex < 0 || xIndex >= MOB_SPRITES.length || yIndex < 0 || yIndex >= MOB_SPRITES[xIndex].length)
			throw new ArrayIndexOutOfBoundsException("Sprite index values out of range: "+xIndex+","+yIndex);
		newSprite = MOB_SPRITES[xIndex][yIndex];
		setSprite(newSprite);
	}
	
	public void setSprite(int x) {
		mobSprite = x; 
	}
	
	public int getSprite() {
		return mobSprite;
	}
	
	public boolean hasMoved() { 
		return mobSprite != -1; 
	}
	
	public void resetMoved() { 
		mobSprite = -1;
	}
}
