package net.varek.rs2d.model;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.mina.common.IoSession;
import net.varek.rs2d.event.DelayedRunnable;
import net.varek.rs2d.net.ActionSender;
import net.varek.rs2d.util.Data;

/**
 * A single player.
 */
public final class Player extends Mob {
	private String username, password,
			lastIP = "0.0.0.0", currentIP = "0.0.0.0";
	private PlayerAppearance appearance;
	private int[] curStats = new int[25], maxStats = new int[25], exps = new int[25], 
			privacySettings = new int[3];
	private Equipment equipment = new Equipment(this);
	private transient boolean reconnecting = false, mapAreaChanged = true;
	private transient byte responseCode;
	private transient long usernameHash;
	private transient IoSession ioSession;
	private transient ActionSender actionSender = new ActionSender(this);
	private transient Map<String,DelayedRunnable> events = new HashMap<String,DelayedRunnable>();
	private transient List<ChatMessage> chatMessages = new ArrayList<ChatMessage>();
	
	public Player(IoSession ios) {
		ioSession = ios;
	}

    public Map<String, DelayedRunnable> getEvents() {
        return events;
    }

    public ActionSender getActionSender() {
        return actionSender;
    }

    public List<ChatMessage> getChatMessages() {
        return chatMessages;
    }

    public Equipment getEquipment() {
        return equipment;
    }

    public void setEquipment(Equipment equipment) {
        this.equipment = equipment;
    }
	
	public int getPrivacySetting(int idx) {
		return privacySettings[idx];
	}
	
	public void setPrivacySetting(int idx, int i) {
		privacySettings[idx] = i;
	}

    public int[] getCurStats() {
        return curStats;
    }
	
	public int getCurStat(int idx) { 
		return curStats[idx];
	}

    public void setCurStats(int[] curStats) {
        this.curStats = curStats;
    }
	
	public void setCurStat(int idx, int s) {
		curStats[idx] = s;
	}

    public int[] getMaxStats() {
        return maxStats;
    }
	
	public int getMaxStat(int idx) { 
		return maxStats[idx];
	}

    public void setMaxStats(int[] maxStats) {
        this.maxStats = maxStats;
    }
	
	public void setMaxStat(int idx, int s) {
		maxStats[idx] = s;
	}

    public int[] getExps() {
        return exps;
    }

    public void setExps(int[] exps) {
        this.exps = exps;
    }

    public String getUsername() {
        return username;
    }
	
	public long getUsernameHash() {
		return usernameHash;
	}
	
    public PlayerAppearance getAppearance() {
        return appearance;
    }

    public void setAppearance(PlayerAppearance appearance) {
        this.appearance = appearance;
    }

    public void setCredentials(String u, String p, boolean r) {
		username = u;
		usernameHash = Data.playerNameToLong(u);
		password = p;
		reconnecting = r;
	}

    public String getLastIP() {
        return lastIP;
    }

    public IoSession getIoSession() {
        return ioSession;
    }

    public void setLastIP(String lastIP) {
        this.lastIP = lastIP;
    }

    public String getCurrentIP() {
        return currentIP;
    }

    public void setCurrentIP(String currentIP) {
        this.currentIP = currentIP;
    }

    public byte getResponseCode() {
        return responseCode;
    }

    public void setResponseCode(byte responseCode) {
        this.responseCode = responseCode;
    }

    public boolean isMapAreaChanged() {
        return mapAreaChanged;
    }

    public void setMapAreaChanged(boolean mapAreaChanged) {
        this.mapAreaChanged = mapAreaChanged;
    }
	
	@Override
	public void setCombatLevel(int c) {
		super.setCombatLevel(c);
		getAppearance().setChanged(true);
	}
	
	@Override
	public void setLocation(Point p) {
		int[] mapArea1 = { location.getRegionX(), location.getRegionY() };
		int[] mapArea2 = { p.getRegionX(), p.getRegionY() };
		if(!Arrays.equals(mapArea1, mapArea2))
			setMapAreaChanged(true);
		super.setLocation(p);
	}
	
	public String toString() { return "['"+username+"',idx="+index+"]";}
}
