package net.varek.rs2d.net;

import net.varek.rs2d.model.Player;
import net.varek.rs2d.packetbuilder.StaticPacketBuilder;

/**
 * These classes simplify network tasks, so you shouldn't have to mess about 
 * with StaticPacketBuilders yourself.
 * Those packets whose purpose I don't understand are left as 'sendUnknownId'.
 * 
 * NOTE: These packets DO NOT modify player state. They simply send data.
 */
public class ActionSender {

	private Player player;
	
	public ActionSender(Player player) {
		this.player = player;
	}
	
	public void sendLoginResponse(int loginResponse, int playerGroup, int flag) {
		player.getIoSession().write(new StaticPacketBuilder().setBare(true)
				.addByte((byte)loginResponse)
				.addByte((byte)playerGroup)
				.addByte((byte)flag).toPacket());
	}
	
	public void sendLoginPacket() {
		player.getIoSession().write(new StaticPacketBuilder().setId(249) //first packet
					.addByteA((byte)1) //1 = members
					.addShortA(player.getIndex()) //player id
					.toPacket());
	}
	
	public void sendMessage(String s) {
		player.getIoSession().write(new StaticPacketBuilder().setId(253).setSize(Packet.Size.VariableByte).addString(s).toPacket());
	}
	
	public void sendPrivacySettings() {
		player.getIoSession().write(new StaticPacketBuilder().setId(206)
				.addByte((byte)player.getPrivacySetting(0))
				.addByte((byte)player.getPrivacySetting(1))
				.addByte((byte)player.getPrivacySetting(2)).toPacket());
	}
	
	public void sendStat(int stat) {
		player.getIoSession().write(new StaticPacketBuilder().setId(134)
				.addByte((byte)stat)
				.addInt1(player.getExps()[stat])
				.addByte((byte)player.getCurStat(stat)).toPacket());
	}
	
	public void sendSideBarInterface(int menuId, int form) {
		player.getIoSession().write(new StaticPacketBuilder().setId(71)
				.addShort(form)
				.addByteA((byte)menuId).toPacket());
	}
	
	public void sendLogout() {
		player.getIoSession().write(new StaticPacketBuilder().setId(109).toPacket());
	}
	
	public void sendUnknown107() {
		player.getIoSession().write(new StaticPacketBuilder().setId(107).toPacket());
	}
	
	public void sendPlayerCommand(int slot, boolean placeOnTop, String command) {
		player.getIoSession().write(new StaticPacketBuilder().setId(104).setSize(Packet.Size.VariableByte)
				.addByte((byte)(-slot))
				.addByteA(placeOnTop ? (byte)0 : (byte)1)
				.addString(command).toPacket());
	}
	
	public void sendLoginWindow(int recoveryChange, boolean memberWarning, int messages, int lastLoginIp, int lastLogin) {
		player.getIoSession().write(new StaticPacketBuilder().setId(176)
				.addByte((byte)-recoveryChange)
				.addShortA(messages)
				.addByte(memberWarning ? (byte)1 : (byte)0)
				.addInt2(lastLoginIp)
				.addShort(lastLogin).toPacket());
	}
	
	public void sendEquipment(int type) {
		int itemId = 0, amount = 0;
		if(player.getEquipment().isEquipped(type)) {
			itemId = player.getEquipment().getId(type);
			amount = player.getEquipment().getAmount(type);
		}
		StaticPacketBuilder equip = new StaticPacketBuilder().setId(34)
				.addShort(1688)
				.addByte((byte)type)
				.addShort(itemId);
		if(amount > 254)
			equip.addByte((byte)255).addInt(amount);
		else equip.addByte((byte)amount);
		player.getIoSession().write(equip.toPacket());
	}
}
