package net.varek.rs2d.net;

import java.util.ArrayList;
import java.util.List;

/**
 * Synchronized packet queue
 */
public class PacketQueue {
	/**
	 * The list of packets in the queue
	 */
	private ArrayList<Packet> packets = new ArrayList<Packet>();

	/**
	 * Adds a packet to the queue
	 */
	public void add(Packet p) {
		synchronized (packets) {
			packets.add(p);
		}
	}

	/**
	 * Returns if there is packets to process
	 */
	public boolean hasPackets() {
		return !packets.isEmpty();
	}

	/**
	 * Returns the packets currently in the list
	 * and removes them from the backing store
	 */
	public List<Packet> getPackets() {
		List<Packet> tmpList;
		synchronized (packets) {
			tmpList = (List<Packet>) packets.clone();
			packets.clear();
		}
		return tmpList;
	}
}
