/*
 * PlayerUpdate.java
 *
 * Created on 17-Dec-2007, 09:22:58
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package net.varek.rs2d.packetbuilder;

import java.util.List;
import org.apache.mina.common.IoSession;
import net.varek.rs2d.model.ChatMessage;
import net.varek.rs2d.model.Equipment;
import net.varek.rs2d.model.Mob;
import net.varek.rs2d.model.Player;
import net.varek.rs2d.net.Packet;
import net.varek.rs2d.util.Logger;

/**
 *
 * @author alex
 */
public class PlayerUpdate implements PacketBuilder {
	
	public Packet getPacket(Player player) {
		StaticPacketBuilder spb = new StaticPacketBuilder();
		spb.setId(81).setSize(Packet.Size.VariableShort);
		boolean updateIsRequired = true;
		
		if(updateIsRequired) {
			spb.addBits(1, 1);
			
			int updateMask = 0;
			List<ChatMessage> pendingMessages = player.getChatMessages();
			if(!pendingMessages.isEmpty())
				updateMask |= 128;
			if(player.getAppearance().hasChanged())
				updateMask |= 16;
			
			/**
			 * self Update type is sent as 2 bits.
			 * Only one of these is sent at once.
			 * 0: No data to send
			 * 1: 3 bits for direction then 1 bit for update required (?)
			 * 2: 3 bits for 1st dir, 3 bits for 2nd dir, then 1 bit for update required (?)
			 * 3: 2 bits for height, 1 bit for whether to discard the walk queue,
			 *		then 1 bit for update required (?), then 7 bits for y position (local probably)
			 *		and 7 bits for x position (local probably).
			 */
			if(player.isMapAreaChanged()) {
				spb.addBits(3, 2); // update type 3
				spb.addBits(player.getLocation().getHeight(), 2); // point height
				spb.addBits(1, 1); // if true, discard the client walking queue
				spb.addBits(updateMask > 0 ? 1 : 0, 1); // is the update required?
				spb.addBits(player.getLocation().getLocalY(), 7);
				spb.addBits(player.getLocation().getLocalX(), 7);
				//Logger.log("sent changed map area");
			} else if(player.hasMoved()) {
				spb.addBits(1, 2); // update type 1
				spb.addBits(Mob.CLIENT_SPRITES[player.getSprite() >> 1], 3); // direction
				spb.addBits(updateMask > 0 ? 1 : 0, 1); // is the update required?
			} else {
				spb.addBits(0, 2); //don't need to update anything
			}
			spb.addBits(0, 8); //we has no players to update
			
			spb.addBits(2047, 11); // magic byte saying we has no players to add
			
			if(updateMask >= 256) {
				updateMask |= 64;
				spb.addByte((byte)(updateMask & 0xff))
						.addByte((byte)(updateMask >> 8));
			} else {
				spb.addByte((byte)updateMask);
			}
			
			if(player.getAppearance().hasChanged()) {
				StaticPacketBuilder appearance = new StaticPacketBuilder();
				appearance.addByte((byte)0) //sex
						.addByte((byte)0); //mask
				
				Equipment e = player.getEquipment();
				if(e.isEquipped(Equipment.HAT))
					appearance.addShort(512 + e.getId(Equipment.HAT));
				else appearance.addByte((byte)0);
				
				if(e.isEquipped(Equipment.CAPE))
					appearance.addShort(512 + e.getId(Equipment.CAPE));
				else appearance.addByte((byte)0);
				
				if(e.isEquipped(Equipment.AMULET))
					appearance.addShort(512 + e.getId(Equipment.AMULET));
				else appearance.addByte((byte)0);
				
				if(e.isEquipped(Equipment.WEAPON))
					appearance.addShort(512 + e.getId(Equipment.WEAPON));
				else appearance.addByte((byte)0);
				
				if(e.isEquipped(Equipment.TORSO))
					appearance.addShort(512 + e.getId(Equipment.TORSO));
				else appearance.addShort(256 + player.getAppearance().getTorso());
				
				if(e.isEquipped(Equipment.SHIELD))
					appearance.addShort(512 + e.getId(Equipment.SHIELD));
				else appearance.addByte((byte)0);
				
				/*if (!Item.isPlate(playerEquipment[playerChest]))
				playerProps.writeWord(0x100+pArms);
				else
				playerProps.writeByte(0);*/
				appearance.addShort(256+player.getAppearance().getArms());
				
				if(e.isEquipped(Equipment.LEGS))
					appearance.addShort(512 + e.getId(Equipment.LEGS));
				else appearance.addShort(256 + player.getAppearance().getLegs());
				
				/*if (!Item.isFullHelm(playerEquipment[playerHat]) && !Item.isFullMask(playerEquipment[playerHat]))
				playerProps.writeWord(0x100 + pHead);		// head
				else
				playerProps.writeByte(0);*/
				appearance.addShort(256+player.getAppearance().getHead());
				
				if(e.isEquipped(Equipment.HANDS))
					appearance.addShort(512 + e.getId(Equipment.HANDS));
				else appearance.addShort(256 + player.getAppearance().getHands());
				
				if(e.isEquipped(Equipment.FEET))
					appearance.addShort(512 + e.getId(Equipment.FEET));
				else appearance.addShort(256 + player.getAppearance().getFeet());
				
				appearance.addByte((byte)0); //end byte?
				
				appearance.addByte((byte)player.getAppearance().getHairColour())
						.addByte((byte)player.getAppearance().getTorsoColour())
						.addByte((byte)player.getAppearance().getLegColour())
						.addByte((byte)player.getAppearance().getFeetColour())
						.addByte((byte)player.getAppearance().getSkinColour());
				
				for(int i : ANIMATION_INDICES)
					appearance.addShort(i);
				
				appearance.addLong(player.getUsernameHash());
				
				appearance.addByte((byte)player.getCombatLevel());
				appearance.addShort((byte)0);
				Packet appearancePacket = appearance.toPacket();
				spb.addByte((byte)(-appearancePacket.getLength()));
				spb.addBytes(appearancePacket.getData(), 0, appearancePacket.getLength());
			}
			
		} else {
			spb.addBits(0, 1);
		}
		Packet p = spb.toPacket();
		return p;
	}
	
	private static final int[] ANIMATION_INDICES = {
		0x328, 0x337, 0x333, 0x334, 0x335, 0x336, 0x338
	};
}
